@@ -29,7 +29,7 @@ class WP_Term_Query {
 	 * Metadata query container.
 	 *
 	 * @since 4.6.0
-	 * @var object WP_Meta_Query
+	 * @var WP_Meta_Query A meta query instance.
 	 */
 	public $meta_query = false;
 
@@ -96,24 +96,29 @@ class WP_Term_Query {
 	 *                                                be limited.
 	 *     @type int|array    $object_ids             Optional. Object ID, or array of object IDs. Results will be
 	 *                                                limited to terms associated with these objects.
-	 *     @type string       $orderby                Field(s) to order terms by. Accepts term fields ('name',
-	 *                                                'slug', 'term_group', 'term_id', 'id', 'description', 'parent'),
-	 *                                                'count' for term taxonomy count, 'include' to match the
-	 *                                                'order' of the $include param, 'slug__in' to match the
-	 *                                                'order' of the $slug param, 'meta_value', 'meta_value_num',
-	 *                                                the value of `$meta_key`, the array keys of `$meta_query`, or
-	 *                                                'none' to omit the ORDER BY clause. Defaults to 'name'.
+	 *     @type string       $orderby                Field(s) to order terms by. Accepts:
+	 *                                                - term fields ('name', 'slug', 'term_group', 'term_id', 'id',
+	 *                                                  'description', 'parent', 'term_order'). Unless `$object_ids`
+	 *                                                  is not empty, 'term_order' is treated the same as 'term_id'.
+	 *                                                - 'count' to use the number of objects associated with the term.
+	 *                                                - 'include' to match the 'order' of the $include param.
+	 *                                                - 'slug__in' to match the 'order' of the $slug param.
+	 *                                                - 'meta_value', 'meta_value_num'.
+	 *                                                - the value of `$meta_key`.
+	 *                                                - the array keys of `$meta_query`.
+	 *                                                - 'none' to omit the ORDER BY clause.
+	 *                                                Defaults to 'name'.
 	 *     @type string       $order                  Whether to order terms in ascending or descending order.
 	 *                                                Accepts 'ASC' (ascending) or 'DESC' (descending).
 	 *                                                Default 'ASC'.
 	 *     @type bool|int     $hide_empty             Whether to hide terms not assigned to any posts. Accepts
 	 *                                                1|true or 0|false. Default 1|true.
-	 *     @type array|string $include                Array or comma/space-separated string of term ids to include.
+	 *     @type array|string $include                Array or comma/space-separated string of term IDs to include.
 	 *                                                Default empty array.
-	 *     @type array|string $exclude                Array or comma/space-separated string of term ids to exclude.
+	 *     @type array|string $exclude                Array or comma/space-separated string of term IDs to exclude.
 	 *                                                If $include is non-empty, $exclude is ignored.
 	 *                                                Default empty array.
-	 *     @type array|string $exclude_tree           Array or comma/space-separated string of term ids to exclude
+	 *     @type array|string $exclude_tree           Array or comma/space-separated string of term IDs to exclude
 	 *                                                along with all of their descendant terms. If $include is
 	 *                                                non-empty, $exclude_tree is ignored. Default empty array.
 	 *     @type int|string   $number                 Maximum number of terms to return. Accepts ''|0 (all) or any
@@ -121,29 +126,33 @@ class WP_Term_Query {
 	 *                                                not return accurate results when coupled with $object_ids.
 	 *                                                See #41796 for details.
 	 *     @type int          $offset                 The number by which to offset the terms query. Default empty.
-	 *     @type string       $fields                 Term fields to query for. Accepts 'all' (returns an array of
-	 *                                                complete term objects), 'all_with_object_id' (returns an
-	 *                                                array of term objects with the 'object_id' param; only works
-	 *                                                when the `$fields` parameter is 'object_ids' ), 'ids'
-	 *                                                (returns an array of ids), 'tt_ids' (returns an array of
-	 *                                                term taxonomy ids), 'id=>parent' (returns an associative
-	 *                                                array with ids as keys, parent term IDs as values), 'names'
-	 *                                                (returns an array of term names), 'count' (returns the number
-	 *                                                of matching terms), 'id=>name' (returns an associative array
-	 *                                                with ids as keys, term names as values), or 'id=>slug'
-	 *                                                (returns an associative array with ids as keys, term slugs
-	 *                                                as values). Default 'all'.
-	 *     @type bool         $count                  Whether to return a term count (true) or array of term objects
-	 *                                                (false). Will take precedence over `$fields` if true.
-	 *                                                Default false.
+	 *     @type string       $fields                 Term fields to query for. Accepts:
+	 *                                                - 'all' Returns an array of complete term objects (`WP_Term[]`).
+	 *                                                - 'all_with_object_id' Returns an array of term objects
+	 *                                                  with the 'object_id' param (`WP_Term[]`). Works only
+	 *                                                  when the `$object_ids` parameter is populated.
+	 *                                                - 'ids' Returns an array of term IDs (`int[]`).
+	 *                                                - 'tt_ids' Returns an array of term taxonomy IDs (`int[]`).
+	 *                                                - 'names' Returns an array of term names (`string[]`).
+	 *                                                - 'slugs' Returns an array of term slugs (`string[]`).
+	 *                                                - 'count' Returns the number of matching terms (`int`).
+	 *                                                - 'id=>parent' Returns an associative array of parent term IDs,
+	 *                                                   keyed by term ID (`int[]`).
+	 *                                                - 'id=>name' Returns an associative array of term names,
+	 *                                                   keyed by term ID (`string[]`).
+	 *                                                - 'id=>slug' Returns an associative array of term slugs,
+	 *                                                   keyed by term ID (`string[]`).
+	 *                                                Default 'all'.
+	 *     @type bool         $count                  Whether to return a term count. If true, will take precedence
+	 *                                                over `$fields`. Default false.
 	 *     @type string|array $name                   Optional. Name or array of names to return term(s) for.
 	 *                                                Default empty.
 	 *     @type string|array $slug                   Optional. Slug or array of slugs to return term(s) for.
 	 *                                                Default empty.
 	 *     @type int|array    $term_taxonomy_id       Optional. Term taxonomy ID, or array of term taxonomy IDs,
 	 *                                                to match when querying terms.
-	 *     @type bool         $hierarchical           Whether to include terms that have non-empty descendants (even
-	 *                                                if $hide_empty is set to true). Default true.
+	 *     @type bool         $hierarchical           Whether to include terms that have non-empty descendants
+	 *                                                (even if $hide_empty is set to true). Default true.
 	 *     @type string       $search                 Search criteria to match terms. Will be SQL-formatted with
 	 *                                                wildcards before and after. Default empty.
 	 *     @type string       $name__like             Retrieve terms with criteria by which a term is LIKE
@@ -172,8 +181,8 @@ class WP_Term_Query {
 	 *                                                Can be used in conjunction with `$meta_value`. Default empty.
 	 *     @type string       $meta_value             Limit terms to those matching a specific metadata value.
 	 *                                                Usually used in conjunction with `$meta_key`. Default empty.
-	 *     @type string       $meta_type              Type of object metadata is for (e.g., comment, post, or user).
-	 *                                                Default empty.
+	 *     @type string       $meta_type              MySQL data type that the `$meta_value` will be CAST to for
+	 *                                                comparisons. Default empty.
 	 *     @type string       $meta_compare           Comparison operator to test the 'meta_value'. Default empty.
 	 * }
 	 */
@@ -238,8 +247,8 @@ class WP_Term_Query {
 		 *
 		 * @since 4.4.0
 		 *
-		 * @param array $defaults   An array of default get_terms() arguments.
-		 * @param array $taxonomies An array of taxonomies.
+		 * @param array    $defaults   An array of default get_terms() arguments.
+		 * @param string[] $taxonomies An array of taxonomy names.
 		 */
 		$this->query_var_defaults = apply_filters( 'get_terms_defaults', $this->query_var_defaults, $taxonomies );
 
@@ -249,16 +258,16 @@ class WP_Term_Query {
 		$query['offset'] = absint( $query['offset'] );
 
 		// 'parent' overrides 'child_of'.
-		if ( 0 < intval( $query['parent'] ) ) {
+		if ( 0 < (int) $query['parent'] ) {
 			$query['child_of'] = false;
 		}
 
-		if ( 'all' == $query['get'] ) {
-			$query['childless'] = false;
-			$query['child_of'] = 0;
-			$query['hide_empty'] = 0;
+		if ( 'all' === $query['get'] ) {
+			$query['childless']    = false;
+			$query['child_of']     = 0;
+			$query['hide_empty']   = 0;
 			$query['hierarchical'] = false;
-			$query['pad_counts'] = false;
+			$query['pad_counts']   = false;
 		}
 
 		$query['taxonomy'] = $taxonomies;
@@ -312,9 +321,9 @@ class WP_Term_Query {
 		 *
 		 * @since 4.6.0
 		 *
-		 * @param WP_Term_Query $this Current instance of WP_Term_Query.
+		 * @param WP_Term_Query $this Current instance of WP_Term_Query (passed by reference).
 		 */
-		do_action( 'pre_get_terms', $this );
+		do_action_ref_array( 'pre_get_terms', array( &$this ) );
 
 		$taxonomies = (array) $args['taxonomy'];
 
@@ -326,24 +335,27 @@ class WP_Term_Query {
 					$has_hierarchical_tax = true;
 				}
 			}
+		} else {
+			// When no taxonomies are provided, assume we have to descend the tree.
+			$has_hierarchical_tax = true;
 		}
 
 		if ( ! $has_hierarchical_tax ) {
 			$args['hierarchical'] = false;
-			$args['pad_counts'] = false;
+			$args['pad_counts']   = false;
 		}
 
 		// 'parent' overrides 'child_of'.
-		if ( 0 < intval( $args['parent'] ) ) {
+		if ( 0 < (int) $args['parent'] ) {
 			$args['child_of'] = false;
 		}
 
-		if ( 'all' == $args['get'] ) {
-			$args['childless'] = false;
-			$args['child_of'] = 0;
-			$args['hide_empty'] = 0;
+		if ( 'all' === $args['get'] ) {
+			$args['childless']    = false;
+			$args['child_of']     = 0;
+			$args['hide_empty']   = 0;
 			$args['hierarchical'] = false;
-			$args['pad_counts'] = false;
+			$args['pad_counts']   = false;
 		}
 
 		/**
@@ -351,8 +363,8 @@ class WP_Term_Query {
 		 *
 		 * @since 3.1.0
 		 *
-		 * @param array $args       An array of get_terms() arguments.
-		 * @param array $taxonomies An array of taxonomies.
+		 * @param array    $args       An array of get_terms() arguments.
+		 * @param string[] $taxonomies An array of taxonomy names.
 		 */
 		$args = apply_filters( 'get_terms_args', $args, $taxonomies );
 
@@ -379,7 +391,12 @@ class WP_Term_Query {
 			}
 
 			if ( ! $in_hierarchy ) {
-				return array();
+				if ( 'count' === $args['fields'] ) {
+					return 0;
+				} else {
+					$this->terms = array();
+					return $this->terms;
+				}
 			}
 		}
 
@@ -388,6 +405,7 @@ class WP_Term_Query {
 		if ( 'term_order' === $_orderby && empty( $this->query_vars['object_ids'] ) ) {
 			$_orderby = 'term_id';
 		}
+
 		$orderby = $this->parse_orderby( $_orderby );
 
 		if ( $orderby ) {
@@ -406,9 +424,9 @@ class WP_Term_Query {
 
 		$inclusions = '';
 		if ( ! empty( $include ) ) {
-			$exclude = '';
+			$exclude      = '';
 			$exclude_tree = '';
-			$inclusions = implode( ',', wp_parse_id_list( $include ) );
+			$inclusions   = implode( ',', wp_parse_id_list( $include ) );
 		}
 
 		if ( ! empty( $inclusions ) ) {
@@ -417,16 +435,19 @@ class WP_Term_Query {
 
 		$exclusions = array();
 		if ( ! empty( $exclude_tree ) ) {
-			$exclude_tree = wp_parse_id_list( $exclude_tree );
+			$exclude_tree      = wp_parse_id_list( $exclude_tree );
 			$excluded_children = $exclude_tree;
 			foreach ( $exclude_tree as $extrunk ) {
 				$excluded_children = array_merge(
 					$excluded_children,
-					(array) get_terms( reset( $taxonomies ), array(
-						'child_of' => intval( $extrunk ),
-						'fields' => 'ids',
-						'hide_empty' => 0
-					) )
+					(array) get_terms(
+						array(
+							'taxonomy'   => reset( $taxonomies ),
+							'child_of'   => (int) $extrunk,
+							'fields'     => 'ids',
+							'hide_empty' => 0,
+						)
+					)
 				);
 			}
 			$exclusions = array_merge( $excluded_children, $exclusions );
@@ -441,7 +462,7 @@ class WP_Term_Query {
 		if ( $childless ) {
 			foreach ( $taxonomies as $_tax ) {
 				$term_hierarchy = _get_term_hierarchy( $_tax );
-				$exclusions = array_merge( array_keys( $term_hierarchy ), $exclusions );
+				$exclusions     = array_merge( array_keys( $term_hierarchy ), $exclusions );
 			}
 		}
 
@@ -456,9 +477,9 @@ class WP_Term_Query {
 		 *
 		 * @since 2.3.0
 		 *
-		 * @param string $exclusions `NOT IN` clause of the terms query.
-		 * @param array  $args       An array of terms query arguments.
-		 * @param array  $taxonomies An array of taxonomies.
+		 * @param string   $exclusions `NOT IN` clause of the terms query.
+		 * @param array    $args       An array of terms query arguments.
+		 * @param string[] $taxonomies An array of taxonomy names.
 		 */
 		$exclusions = apply_filters( 'list_terms_exclusions', $exclusions, $args, $taxonomies );
 
@@ -485,10 +506,10 @@ class WP_Term_Query {
 			( is_string( $args['slug'] ) && 0 !== strlen( $args['slug'] ) )
 		) {
 			if ( is_array( $args['slug'] ) ) {
-				$slug = array_map( 'sanitize_title', $args['slug'] );
+				$slug                               = array_map( 'sanitize_title', $args['slug'] );
 				$this->sql_clauses['where']['slug'] = "t.slug IN ('" . implode( "', '", $slug ) . "')";
 			} else {
-				$slug = sanitize_title( $args['slug'] );
+				$slug                               = sanitize_title( $args['slug'] );
 				$this->sql_clauses['where']['slug'] = "t.slug = '$slug'";
 			}
 		}
@@ -498,16 +519,16 @@ class WP_Term_Query {
 				$tt_ids = implode( ',', array_map( 'intval', $args['term_taxonomy_id'] ) );
 				$this->sql_clauses['where']['term_taxonomy_id'] = "tt.term_taxonomy_id IN ({$tt_ids})";
 			} else {
-				$this->sql_clauses['where']['term_taxonomy_id'] = $wpdb->prepare( "tt.term_taxonomy_id = %d", $args['term_taxonomy_id'] );
+				$this->sql_clauses['where']['term_taxonomy_id'] = $wpdb->prepare( 'tt.term_taxonomy_id = %d', $args['term_taxonomy_id'] );
 			}
 		}
 
 		if ( ! empty( $args['name__like'] ) ) {
-			$this->sql_clauses['where']['name__like'] = $wpdb->prepare( "t.name LIKE %s", '%' . $wpdb->esc_like( $args['name__like'] ) . '%' );
+			$this->sql_clauses['where']['name__like'] = $wpdb->prepare( 't.name LIKE %s', '%' . $wpdb->esc_like( $args['name__like'] ) . '%' );
 		}
 
 		if ( ! empty( $args['description__like'] ) ) {
-			$this->sql_clauses['where']['description__like'] = $wpdb->prepare( "tt.description LIKE %s", '%' . $wpdb->esc_like( $args['description__like'] ) . '%' );
+			$this->sql_clauses['where']['description__like'] = $wpdb->prepare( 'tt.description LIKE %s', '%' . $wpdb->esc_like( $args['description__like'] ) . '%' );
 		}
 
 		if ( ! empty( $args['object_ids'] ) ) {
@@ -516,7 +537,7 @@ class WP_Term_Query {
 				$object_ids = array( $object_ids );
 			}
 
-			$object_ids = implode( ', ', array_map( 'intval', $object_ids ) );
+			$object_ids                               = implode( ', ', array_map( 'intval', $object_ids ) );
 			$this->sql_clauses['where']['object_ids'] = "tr.object_id IN ($object_ids)";
 		}
 
@@ -529,15 +550,15 @@ class WP_Term_Query {
 		}
 
 		if ( '' !== $parent ) {
-			$parent = (int) $parent;
+			$parent                               = (int) $parent;
 			$this->sql_clauses['where']['parent'] = "tt.parent = '$parent'";
 		}
 
 		$hierarchical = $args['hierarchical'];
-		if ( 'count' == $args['fields'] ) {
+		if ( 'count' === $args['fields'] ) {
 			$hierarchical = false;
 		}
-		if ( $args['hide_empty'] && !$hierarchical ) {
+		if ( $args['hide_empty'] && ! $hierarchical ) {
 			$this->sql_clauses['where']['count'] = 'tt.count > 0';
 		}
 
@@ -555,33 +576,32 @@ class WP_Term_Query {
 			$limits = '';
 		}
 
-
 		if ( ! empty( $args['search'] ) ) {
 			$this->sql_clauses['where']['search'] = $this->get_search_sql( $args['search'] );
 		}
 
 		// Meta query support.
-		$join = '';
+		$join     = '';
 		$distinct = '';
 
 		// Reparse meta_query query_vars, in case they were modified in a 'pre_get_terms' callback.
 		$this->meta_query->parse_query_vars( $this->query_vars );
-		$mq_sql = $this->meta_query->get_sql( 'term', 't', 'term_id' );
+		$mq_sql       = $this->meta_query->get_sql( 'term', 't', 'term_id' );
 		$meta_clauses = $this->meta_query->get_clauses();
 
 		if ( ! empty( $meta_clauses ) ) {
-			$join .= $mq_sql['join'];
+			$join                                    .= $mq_sql['join'];
 			$this->sql_clauses['where']['meta_query'] = preg_replace( '/^\s*AND\s*/', '', $mq_sql['where'] );
-			$distinct .= "DISTINCT";
+			$distinct                                .= 'DISTINCT';
 
 		}
 
 		$selects = array();
 		switch ( $args['fields'] ) {
 			case 'all':
-			case 'all_with_object_id' :
-			case 'tt_ids' :
-			case 'slugs' :
+			case 'all_with_object_id':
+			case 'tt_ids':
+			case 'slugs':
 				$selects = array( 't.*', 'tt.*' );
 				if ( 'all_with_object_id' === $args['fields'] && ! empty( $args['object_ids'] ) ) {
 					$selects[] = 'tr.object_id';
@@ -596,14 +616,14 @@ class WP_Term_Query {
 				break;
 			case 'count':
 				$orderby = '';
-				$order = '';
+				$order   = '';
 				$selects = array( 'COUNT(*)' );
 				break;
 			case 'id=>name':
-				$selects = array( 't.term_id', 't.name', 'tt.count', 'tt.taxonomy' );
+				$selects = array( 't.term_id', 't.name', 'tt.parent', 'tt.count', 'tt.taxonomy' );
 				break;
 			case 'id=>slug':
-				$selects = array( 't.term_id', 't.slug', 'tt.count', 'tt.taxonomy' );
+				$selects = array( 't.term_id', 't.slug', 'tt.parent', 'tt.count', 'tt.taxonomy' );
 				break;
 		}
 
@@ -621,9 +641,9 @@ class WP_Term_Query {
 		 *
 		 * @since 2.8.0
 		 *
-		 * @param array $selects    An array of fields to select for the terms query.
-		 * @param array $args       An array of term query arguments.
-		 * @param array $taxonomies An array of taxonomies.
+		 * @param string[] $selects    An array of fields to select for the terms query.
+		 * @param array    $args       An array of term query arguments.
+		 * @param string[] $taxonomies An array of taxonomy names.
 		 */
 		$fields = implode( ', ', apply_filters( 'get_terms_fields', $selects, $args, $taxonomies ) );
 
@@ -640,19 +660,19 @@ class WP_Term_Query {
 		 *
 		 * @since 3.1.0
 		 *
-		 * @param array $pieces     Terms query SQL clauses.
-		 * @param array $taxonomies An array of taxonomies.
-		 * @param array $args       An array of terms query arguments.
+		 * @param string[] $pieces     Array of query SQL clauses.
+		 * @param string[] $taxonomies An array of taxonomy names.
+		 * @param array    $args       An array of term query arguments.
 		 */
 		$clauses = apply_filters( 'terms_clauses', compact( 'fields', 'join', 'where', 'distinct', 'orderby', 'order', 'limits' ), $taxonomies, $args );
 
-		$fields = isset( $clauses[ 'fields' ] ) ? $clauses[ 'fields' ] : '';
-		$join = isset( $clauses[ 'join' ] ) ? $clauses[ 'join' ] : '';
-		$where = isset( $clauses[ 'where' ] ) ? $clauses[ 'where' ] : '';
-		$distinct = isset( $clauses[ 'distinct' ] ) ? $clauses[ 'distinct' ] : '';
-		$orderby = isset( $clauses[ 'orderby' ] ) ? $clauses[ 'orderby' ] : '';
-		$order = isset( $clauses[ 'order' ] ) ? $clauses[ 'order' ] : '';
-		$limits = isset( $clauses[ 'limits' ] ) ? $clauses[ 'limits' ] : '';
+		$fields   = isset( $clauses['fields'] ) ? $clauses['fields'] : '';
+		$join     = isset( $clauses['join'] ) ? $clauses['join'] : '';
+		$where    = isset( $clauses['where'] ) ? $clauses['where'] : '';
+		$distinct = isset( $clauses['distinct'] ) ? $clauses['distinct'] : '';
+		$orderby  = isset( $clauses['orderby'] ) ? $clauses['orderby'] : '';
+		$order    = isset( $clauses['order'] ) ? $clauses['order'] : '';
+		$limits   = isset( $clauses['limits'] ) ? $clauses['limits'] : '';
 
 		if ( $where ) {
 			$where = "WHERE $where";
@@ -665,28 +685,48 @@ class WP_Term_Query {
 
 		$this->request = "{$this->sql_clauses['select']} {$this->sql_clauses['from']} {$where} {$this->sql_clauses['orderby']} {$this->sql_clauses['limits']}";
 
+		$this->terms = null;
+
+		/**
+		 * Filters the terms array before the query takes place.
+		 *
+		 * Return a non-null value to bypass WordPress' default term queries.
+		 *
+		 * @since 5.3.0
+		 *
+		 * @param array|null    $terms Return an array of term data to short-circuit WP's term query,
+		 *                             or null to allow WP queries to run normally.
+		 * @param WP_Term_Query $this  The WP_Term_Query instance, passed by reference.
+		 */
+		$this->terms = apply_filters_ref_array( 'terms_pre_query', array( $this->terms, &$this ) );
+
+		if ( null !== $this->terms ) {
+			return $this->terms;
+		}
+
 		// $args can be anything. Only use the args defined in defaults to compute the key.
-		$key = md5( serialize( wp_array_slice_assoc( $args, array_keys( $this->query_var_defaults ) ) ) . serialize( $taxonomies ) . $this->request );
+		$key          = md5( serialize( wp_array_slice_assoc( $args, array_keys( $this->query_var_defaults ) ) ) . serialize( $taxonomies ) . $this->request );
 		$last_changed = wp_cache_get_last_changed( 'terms' );
-		$cache_key = "get_terms:$key:$last_changed";
-		$cache = wp_cache_get( $cache_key, 'terms' );
+		$cache_key    = "get_terms:$key:$last_changed";
+		$cache        = wp_cache_get( $cache_key, 'terms' );
 		if ( false !== $cache ) {
 			if ( 'all' === $_fields || 'all_with_object_id' === $_fields ) {
-				$cache = array_map( 'get_term', $cache );
+				$cache = $this->populate_terms( $cache );
 			}
 
 			$this->terms = $cache;
 			return $this->terms;
 		}
 
-		if ( 'count' == $_fields ) {
+		if ( 'count' === $_fields ) {
 			$count = $wpdb->get_var( $this->request );
 			wp_cache_set( $cache_key, $count, 'terms' );
 			return $count;
 		}
 
 		$terms = $wpdb->get_results( $this->request );
-		if ( 'all' == $_fields || 'all_with_object_id' === $_fields ) {
+
+		if ( 'all' === $_fields || 'all_with_object_id' === $_fields ) {
 			update_term_cache( $terms );
 		}
 
@@ -711,7 +751,7 @@ class WP_Term_Query {
 		}
 
 		// Update term counts to include children.
-		if ( $args['pad_counts'] && 'all' == $_fields ) {
+		if ( $args['pad_counts'] && 'all' === $_fields ) {
 			foreach ( $taxonomies as $_tax ) {
 				_pad_term_counts( $terms, $_tax );
 			}
@@ -743,46 +783,47 @@ class WP_Term_Query {
 		 * `$fields` is 'all_with_object_id', but should otherwise be
 		 * removed.
 		 */
-		if ( ! empty( $args['object_ids'] ) && 'all_with_object_id' != $_fields ) {
-			$_tt_ids = $_terms = array();
+		if ( ! empty( $args['object_ids'] ) && 'all_with_object_id' !== $_fields ) {
+			$_tt_ids = array();
+			$_terms  = array();
 			foreach ( $terms as $term ) {
 				if ( isset( $_tt_ids[ $term->term_id ] ) ) {
 					continue;
 				}
 
 				$_tt_ids[ $term->term_id ] = 1;
-				$_terms[] = $term;
+				$_terms[]                  = $term;
 			}
 
 			$terms = $_terms;
 		}
 
 		$_terms = array();
-		if ( 'id=>parent' == $_fields ) {
+		if ( 'id=>parent' === $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[ $term->term_id ] = $term->parent;
 			}
-		} elseif ( 'ids' == $_fields ) {
+		} elseif ( 'ids' === $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[] = (int) $term->term_id;
 			}
-		} elseif ( 'tt_ids' == $_fields ) {
+		} elseif ( 'tt_ids' === $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[] = (int) $term->term_taxonomy_id;
 			}
-		} elseif ( 'names' == $_fields ) {
+		} elseif ( 'names' === $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[] = $term->name;
 			}
-		} elseif ( 'slugs' == $_fields ) {
+		} elseif ( 'slugs' === $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[] = $term->slug;
 			}
-		} elseif ( 'id=>name' == $_fields ) {
+		} elseif ( 'id=>name' === $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[ $term->term_id ] = $term->name;
 			}
-		} elseif ( 'id=>slug' == $_fields ) {
+		} elseif ( 'id=>slug' === $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[ $term->term_id ] = $term->slug;
 			}
@@ -804,7 +845,7 @@ class WP_Term_Query {
 		wp_cache_add( $cache_key, $terms, 'terms', DAY_IN_SECONDS );
 
 		if ( 'all' === $_fields || 'all_with_object_id' === $_fields ) {
-			$terms = array_map( 'get_term', $terms );
+			$terms = $this->populate_terms( $terms );
 		}
 
 		$this->terms = $terms;
@@ -822,7 +863,7 @@ class WP_Term_Query {
 	 * @return string|false Value to used in the ORDER clause. False otherwise.
 	 */
 	protected function parse_orderby( $orderby_raw ) {
-		$_orderby = strtolower( $orderby_raw );
+		$_orderby           = strtolower( $orderby_raw );
 		$maybe_orderby_meta = false;
 
 		if ( in_array( $_orderby, array( 'term_id', 'name', 'slug', 'term_group' ), true ) ) {
@@ -831,15 +872,15 @@ class WP_Term_Query {
 			$orderby = "tt.$_orderby";
 		} elseif ( 'term_order' === $_orderby ) {
 			$orderby = 'tr.term_order';
-		} elseif ( 'include' == $_orderby && ! empty( $this->query_vars['include'] ) ) {
+		} elseif ( 'include' === $_orderby && ! empty( $this->query_vars['include'] ) ) {
 			$include = implode( ',', wp_parse_id_list( $this->query_vars['include'] ) );
 			$orderby = "FIELD( t.term_id, $include )";
-		} elseif ( 'slug__in' == $_orderby && ! empty( $this->query_vars['slug'] ) && is_array( $this->query_vars['slug'] ) ) {
-			$slugs = implode( "', '", array_map( 'sanitize_title_for_query', $this->query_vars['slug'] ) );
+		} elseif ( 'slug__in' === $_orderby && ! empty( $this->query_vars['slug'] ) && is_array( $this->query_vars['slug'] ) ) {
+			$slugs   = implode( "', '", array_map( 'sanitize_title_for_query', $this->query_vars['slug'] ) );
 			$orderby = "FIELD( t.slug, '" . $slugs . "')";
-		} elseif ( 'none' == $_orderby ) {
+		} elseif ( 'none' === $_orderby ) {
 			$orderby = '';
-		} elseif ( empty( $_orderby ) || 'id' == $_orderby || 'term_id' === $_orderby ) {
+		} elseif ( empty( $_orderby ) || 'id' === $_orderby || 'term_id' === $_orderby ) {
 			$orderby = 't.term_id';
 		} else {
 			$orderby = 't.name';
@@ -853,9 +894,9 @@ class WP_Term_Query {
 		 *
 		 * @since 2.8.0
 		 *
-		 * @param string $orderby    `ORDERBY` clause of the terms query.
-		 * @param array  $args       An array of terms query arguments.
-		 * @param array  $taxonomies An array of taxonomies.
+		 * @param string   $orderby    `ORDERBY` clause of the terms query.
+		 * @param array    $args       An array of term query arguments.
+		 * @param string[] $taxonomies An array of taxonomy names.
 		 */
 		$orderby = apply_filters( 'get_terms_orderby', $orderby, $this->query_vars, $this->query_vars['taxonomy'] );
 
@@ -888,12 +929,12 @@ class WP_Term_Query {
 			return $orderby;
 		}
 
-		$allowed_keys = array();
-		$primary_meta_key = null;
+		$allowed_keys       = array();
+		$primary_meta_key   = null;
 		$primary_meta_query = reset( $meta_clauses );
 		if ( ! empty( $primary_meta_query['key'] ) ) {
 			$primary_meta_key = $primary_meta_query['key'];
-			$allowed_keys[] = $primary_meta_key;
+			$allowed_keys[]   = $primary_meta_key;
 		}
 		$allowed_keys[] = 'meta_value';
 		$allowed_keys[] = 'meta_value_num';
@@ -903,7 +944,7 @@ class WP_Term_Query {
 			return $orderby;
 		}
 
-		switch( $orderby_raw ) {
+		switch ( $orderby_raw ) {
 			case $primary_meta_key:
 			case 'meta_value':
 				if ( ! empty( $primary_meta_query['type'] ) ) {
@@ -921,7 +962,7 @@ class WP_Term_Query {
 				if ( array_key_exists( $orderby_raw, $meta_clauses ) ) {
 					// $orderby corresponds to a meta_query clause.
 					$meta_clause = $meta_clauses[ $orderby_raw ];
-					$orderby = "CAST({$meta_clause['alias']}.meta_value AS {$meta_clause['cast']})";
+					$orderby     = "CAST({$meta_clause['alias']}.meta_value AS {$meta_clause['cast']})";
 				}
 				break;
 		}
@@ -966,4 +1007,31 @@ class WP_Term_Query {
 
 		return $wpdb->prepare( '((t.name LIKE %s) OR (t.slug LIKE %s))', $like, $like );
 	}
+
+	/**
+	 * Creates an array of term objects from an array of term IDs.
+	 *
+	 * Also discards invalid term objects.
+	 *
+	 * @since 4.9.8
+	 *
+	 * @param array $term_ids Term IDs.
+	 * @return array
+	 */
+	protected function populate_terms( $term_ids ) {
+		$terms = array();
+
+		if ( ! is_array( $term_ids ) ) {
+			return $terms;
+		}
+
+		foreach ( $term_ids as $key => $term_id ) {
+			$term = get_term( $term_id );
+			if ( $term instanceof WP_Term ) {
+				$terms[ $key ] = $term;
+			}
+		}
+
+		return $terms;
+	}
 }
