@@ -5,19 +5,19 @@
  * The query API attempts to get which part of WordPress the user is on. It
  * also provides functionality for getting URL query information.
  *
- * @link https://codex.wordpress.org/The_Loop More information on The Loop.
+ * @link https://developer.wordpress.org/themes/basics/the-loop/ More information on The Loop.
  *
  * @package WordPress
  * @subpackage Query
  */
 
 /**
- * Retrieve variable in the WP_Query class.
+ * Retrieves the value of a query variable in the WP_Query class.
  *
  * @since 1.5.0
  * @since 3.9.0 The `$default` argument was introduced.
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
  * @param string $var       The variable key to retrieve.
  * @param mixed  $default   Optional. Value to return if the query variable is not set. Default empty.
@@ -29,15 +29,15 @@ function get_query_var( $var, $default = '' ) {
 }
 
 /**
- * Retrieve the currently-queried object.
+ * Retrieves the currently queried object.
  *
  * Wrapper for WP_Query::get_queried_object().
  *
  * @since 3.1.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return object Queried object.
+ * @return WP_Term|WP_Post_Type|WP_Post|WP_User|null The queried object.
  */
 function get_queried_object() {
 	global $wp_query;
@@ -45,13 +45,13 @@ function get_queried_object() {
 }
 
 /**
- * Retrieve ID of the current queried object.
+ * Retrieves the ID of the currently queried object.
  *
  * Wrapper for WP_Query::get_queried_object_id().
  *
  * @since 3.1.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
  * @return int ID of the queried object.
  */
@@ -61,11 +61,11 @@ function get_queried_object_id() {
 }
 
 /**
- * Set query variable.
+ * Sets the value of a query variable in the WP_Query class.
  *
  * @since 2.2.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
  * @param string $var   Query variable key.
  * @param mixed  $value Query variable value.
@@ -88,14 +88,14 @@ function set_query_var( $var, $value ) {
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
  * @param array|string $query Array or string of WP_Query arguments.
- * @return array List of post objects.
+ * @return WP_Post[]|int[] Array of post objects or post IDs.
  */
-function query_posts($query) {
+function query_posts( $query ) {
 	$GLOBALS['wp_query'] = new WP_Query();
-	return $GLOBALS['wp_query']->query($query);
+	return $GLOBALS['wp_query']->query( $query );
 }
 
 /**
@@ -107,7 +107,7 @@ function query_posts($query) {
  *
  * @since 2.3.0
  *
- * @global WP_Query $wp_query     Global WP_Query instance.
+ * @global WP_Query $wp_query     WordPress Query object.
  * @global WP_Query $wp_the_query Copy of the global WP_Query instance created during wp_reset_query().
  */
 function wp_reset_query() {
@@ -121,7 +121,7 @@ function wp_reset_query() {
  *
  * @since 3.0.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  */
 function wp_reset_postdata() {
 	global $wp_query;
@@ -138,17 +138,24 @@ function wp_reset_postdata() {
 /**
  * Determines whether the query is for an existing archive page.
  *
- * Month, Year, Category, Author, Post Type archive...
- * 
+ * Archive pages include category, tag, author, date, custom post type,
+ * and custom taxonomy based archives.
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @see is_category()
+ * @see is_tag()
+ * @see is_author()
+ * @see is_date()
+ * @see is_post_type_archive()
+ * @see is_tax()
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for an existing archive page.
  */
 function is_archive() {
 	global $wp_query;
@@ -163,17 +170,18 @@ function is_archive() {
 
 /**
  * Determines whether the query is for an existing post type archive page.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 3.1.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param string|array $post_types Optional. Post type or array of posts types to check against.
- * @return bool
+ * @param string|string[] $post_types Optional. Post type or array of posts types
+ *                                    to check against. Default empty.
+ * @return bool Whether the query is for an existing post type archive page.
  */
 function is_post_type_archive( $post_types = '' ) {
 	global $wp_query;
@@ -188,17 +196,18 @@ function is_post_type_archive( $post_types = '' ) {
 
 /**
  * Determines whether the query is for an existing attachment page.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 2.0.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param int|string|array|object $attachment Attachment ID, title, slug, or array of such.
- * @return bool
+ * @param int|string|int[]|string[] $attachment Optional. Attachment ID, title, slug, or array of such
+ *                                              to check against. Default empty.
+ * @return bool Whether the query is for an existing attachment page.
  */
 function is_attachment( $attachment = '' ) {
 	global $wp_query;
@@ -216,17 +225,18 @@ function is_attachment( $attachment = '' ) {
  *
  * If the $author parameter is specified, this function will additionally
  * check if the query is for one of the authors specified.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param mixed $author Optional. User ID, nickname, nicename, or array of User IDs, nicknames, and nicenames
- * @return bool
+ * @param int|string|int[]|string[] $author Optional. User ID, nickname, nicename, or array of such
+ *                                          to check against. Default empty.
+ * @return bool Whether the query is for an existing author archive page.
  */
 function is_author( $author = '' ) {
 	global $wp_query;
@@ -244,17 +254,18 @@ function is_author( $author = '' ) {
  *
  * If the $category parameter is specified, this function will additionally
  * check if the query is for one of the categories specified.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param mixed $category Optional. Category ID, name, slug, or array of Category IDs, names, and slugs.
- * @return bool
+ * @param int|string|int[]|string[] $category Optional. Category ID, name, slug, or array of such
+ *                                            to check against. Default empty.
+ * @return bool Whether the query is for an existing category archive page.
  */
 function is_category( $category = '' ) {
 	global $wp_query;
@@ -272,17 +283,18 @@ function is_category( $category = '' ) {
  *
  * If the $tag parameter is specified, this function will additionally
  * check if the query is for one of the tags specified.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
  *
  * @since 2.3.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param mixed $tag Optional. Tag ID, name, slug, or array of Tag IDs, names, and slugs.
- * @return bool
+ * @param int|string|int[]|string[] $tag Optional. Tag ID, name, slug, or array of such
+ *                                       to check against. Default empty.
+ * @return bool Whether the query is for an existing tag archive page.
  */
 function is_tag( $tag = '' ) {
 	global $wp_query;
@@ -304,18 +316,22 @@ function is_tag( $tag = '' ) {
  * If the $term parameter is specified in addition to the $taxonomy parameter,
  * this function will additionally check if the query is for one of the terms
  * specified.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
  *
  * @since 2.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param string|array     $taxonomy Optional. Taxonomy slug or slugs.
- * @param int|string|array $term     Optional. Term ID, name, slug or array of Term IDs, names, and slugs.
- * @return bool True for custom taxonomy archive pages, false for built-in taxonomies (category and tag archives).
+ * @param string|string[]           $taxonomy Optional. Taxonomy slug or slugs to check against.
+ *                                            Default empty.
+ * @param int|string|int[]|string[] $term     Optional. Term ID, name, slug, or array of such
+ *                                            to check against. Default empty.
+ * @return bool Whether the query is for an existing custom taxonomy archive page.
+ *              True for custom taxonomy archive pages, false for built-in taxonomies
+ *              (category and tag archives).
  */
 function is_tax( $taxonomy = '', $term = '' ) {
 	global $wp_query;
@@ -330,16 +346,16 @@ function is_tax( $taxonomy = '', $term = '' ) {
 
 /**
  * Determines whether the query is for an existing date archive.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for an existing date archive.
  */
 function is_date() {
 	global $wp_query;
@@ -355,15 +371,17 @@ function is_date() {
 /**
  * Determines whether the query is for an existing day archive.
  *
+ * A conditional check to test whether the page is a date-based archive page displaying posts for the current day.
+ *
  * For more information on this and similar theme functions, check out
  * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for an existing day archive.
  */
 function is_day() {
 	global $wp_query;
@@ -378,17 +396,18 @@ function is_day() {
 
 /**
  * Determines whether the query is for a feed.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param string|array $feeds Optional feed types to check.
- * @return bool
+ * @param string|string[] $feeds Optional. Feed type or array of feed types
+ *                                         to check against. Default empty.
+ * @return bool Whether the query is for a feed.
  */
 function is_feed( $feeds = '' ) {
 	global $wp_query;
@@ -406,9 +425,9 @@ function is_feed( $feeds = '' ) {
  *
  * @since 3.0.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for a comments feed.
  */
 function is_comment_feed() {
 	global $wp_query;
@@ -432,16 +451,16 @@ function is_comment_feed() {
  * true when viewing that page.
  *
  * Otherwise the same as @see is_home()
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 2.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool True, if front of site.
+ * @return bool Whether the query is for the front page of the site.
  */
 function is_front_page() {
 	global $wp_query;
@@ -464,17 +483,17 @@ function is_front_page() {
  *
  * If a static page is set for the front page of the site, this function will return true only
  * on the page you set as the "Posts page".
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
  * @see is_front_page()
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool True if blog view homepage, otherwise false.
+ * @return bool Whether the query is for the blog homepage.
  */
 function is_home() {
 	global $wp_query;
@@ -488,17 +507,47 @@ function is_home() {
 }
 
 /**
- * Determines whether the query is for an existing month archive.
- * 
+ * Determines whether the query is for the Privacy Policy page.
+ *
+ * The Privacy Policy page is the page that shows the Privacy Policy content of the site.
+ *
+ * is_privacy_policy() is dependent on the site's "Change your Privacy Policy page" Privacy Settings 'wp_page_for_privacy_policy'.
+ *
+ * This function will return true only on the page you set as the "Privacy Policy page".
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
+ * @since 5.2.0
+ *
+ * @global WP_Query $wp_query WordPress Query object.
+ *
+ * @return bool Whether the query is for the Privacy Policy page.
+ */
+function is_privacy_policy() {
+	global $wp_query;
+
+	if ( ! isset( $wp_query ) ) {
+		_doing_it_wrong( __FUNCTION__, __( 'Conditional query tags do not work before the query is run. Before then, they always return false.' ), '3.1.0' );
+		return false;
+	}
+
+	return $wp_query->is_privacy_policy();
+}
+
+/**
+ * Determines whether the query is for an existing month archive.
+ *
+ * For more information on this and similar theme functions, check out
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
+ * Conditional Tags} article in the Theme Developer Handbook.
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for an existing month archive.
  */
 function is_month() {
 	global $wp_query;
@@ -516,19 +565,19 @@ function is_month() {
  *
  * If the $page parameter is specified, this function will additionally
  * check if the query is for one of the pages specified.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
- * @see is_single()
- * @see is_singular()
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @see is_single()
+ * @see is_singular()
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param int|string|array $page Optional. Page ID, title, slug, or array of such. Default empty.
+ * @param int|string|int[]|string[] $page Optional. Page ID, title, slug, or array of such
+ *                                        to check against. Default empty.
  * @return bool Whether the query is for an existing single page.
  */
 function is_page( $page = '' ) {
@@ -543,17 +592,17 @@ function is_page( $page = '' ) {
 }
 
 /**
- * Determines whether the query is for paged results and not for the first page.
- * 
+ * Determines whether the query is for a paged result and not for the first page.
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for a paged result.
  */
 function is_paged() {
 	global $wp_query;
@@ -568,16 +617,16 @@ function is_paged() {
 
 /**
  * Determines whether the query is for a post or page preview.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 2.0.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for a post or page preview.
  */
 function is_preview() {
 	global $wp_query;
@@ -591,13 +640,13 @@ function is_preview() {
 }
 
 /**
- * Is the query for the robots file?
+ * Is the query for the robots.txt file?
  *
  * @since 2.1.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for the robots.txt file.
  */
 function is_robots() {
 	global $wp_query;
@@ -610,18 +659,38 @@ function is_robots() {
 	return $wp_query->is_robots();
 }
 
+/**
+ * Is the query for the favicon.ico file?
+ *
+ * @since 5.4.0
+ *
+ * @global WP_Query $wp_query WordPress Query object.
+ *
+ * @return bool Whether the query is for the favicon.ico file.
+ */
+function is_favicon() {
+	global $wp_query;
+
+	if ( ! isset( $wp_query ) ) {
+		_doing_it_wrong( __FUNCTION__, __( 'Conditional query tags do not work before the query is run. Before then, they always return false.' ), '3.1.0' );
+		return false;
+	}
+
+	return $wp_query->is_favicon();
+}
+
 /**
  * Determines whether the query is for a search.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for a search.
  */
 function is_search() {
 	global $wp_query;
@@ -641,19 +710,19 @@ function is_search() {
  *
  * If the $post parameter is specified, this function will additionally
  * check if the query is for one of the Posts specified.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
- * @see is_page()
- * @see is_singular()
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @see is_page()
+ * @see is_singular()
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param int|string|array $post Optional. Post ID, title, slug, or array of such. Default empty.
+ * @param int|string|int[]|string[] $post Optional. Post ID, title, slug, or array of such
+ *                                        to check against. Default empty.
  * @return bool Whether the query is for an existing single post.
  */
 function is_single( $post = '' ) {
@@ -673,20 +742,21 @@ function is_single( $post = '' ) {
  *
  * If the $post_types parameter is specified, this function will additionally
  * check if the query is for one of the Posts Types specified.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
- * @see is_page()
- * @see is_single()
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @see is_page()
+ * @see is_single()
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @param string|array $post_types Optional. Post type or array of post types. Default empty.
- * @return bool Whether the query is for an existing single post of any of the given post types.
+ * @param string|string[] $post_types Optional. Post type or array of post types
+ *                                    to check against. Default empty.
+ * @return bool Whether the query is for an existing single post
+ *              or any of the given post types.
  */
 function is_singular( $post_types = '' ) {
 	global $wp_query;
@@ -701,16 +771,16 @@ function is_singular( $post_types = '' ) {
 
 /**
  * Determines whether the query is for a specific time.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for a specific time.
  */
 function is_time() {
 	global $wp_query;
@@ -725,16 +795,16 @@ function is_time() {
 
 /**
  * Determines whether the query is for a trackback endpoint call.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for a trackback endpoint call.
  */
 function is_trackback() {
 	global $wp_query;
@@ -749,16 +819,16 @@ function is_trackback() {
 
 /**
  * Determines whether the query is for an existing year archive.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is for an existing year archive.
  */
 function is_year() {
 	global $wp_query;
@@ -773,16 +843,16 @@ function is_year() {
 
 /**
  * Determines whether the query has resulted in a 404 (returns no results).
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is a 404 error.
  */
 function is_404() {
 	global $wp_query;
@@ -800,9 +870,9 @@ function is_404() {
  *
  * @since 4.4.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool Whether we're in an embedded post or not.
+ * @return bool Whether the query is for an embedded post.
  */
 function is_embed() {
 	global $wp_query;
@@ -817,21 +887,21 @@ function is_embed() {
 
 /**
  * Determines whether the query is the main query.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
- * 
+ *
  * @since 3.3.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool Whether the query is the main query.
  */
 function is_main_query() {
 	if ( 'pre_get_posts' === current_filter() ) {
 		$message = sprintf(
-			/* translators: 1: pre_get_posts 2: WP_Query->is_main_query() 3: is_main_query() 4: link to codex is_main_query() page. */
+			/* translators: 1: pre_get_posts, 2: WP_Query->is_main_query(), 3: is_main_query(), 4: Link to codex is_main_query() page. */
 			__( 'In %1$s, use the %2$s method, not the %3$s function. See %4$s.' ),
 			'<code>pre_get_posts</code>',
 			'<code>WP_Query->is_main_query()</code>',
@@ -850,13 +920,13 @@ function is_main_query() {
  */
 
 /**
- * Whether current WordPress query has results to loop over.
+ * Determines whether current WordPress query has posts to loop over.
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool True if posts are available, false if end of the loop.
  */
 function have_posts() {
 	global $wp_query;
@@ -865,14 +935,14 @@ function have_posts() {
 
 /**
  * Determines whether the caller is in the Loop.
- * 
+ *
  * For more information on this and similar theme functions, check out
- * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/ 
+ * the {@link https://developer.wordpress.org/themes/basics/conditional-tags/
  * Conditional Tags} article in the Theme Developer Handbook.
  *
  * @since 2.0.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
  * @return bool True if caller is within loop, false if loop hasn't started or ended.
  */
@@ -886,7 +956,7 @@ function in_the_loop() {
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  */
 function rewind_posts() {
 	global $wp_query;
@@ -898,7 +968,7 @@ function rewind_posts() {
  *
  * @since 1.5.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  */
 function the_post() {
 	global $wp_query;
@@ -910,13 +980,13 @@ function the_post() {
  */
 
 /**
- * Whether there are comments to loop over.
+ * Determines whether current WordPress query has comments to loop over.
  *
  * @since 2.2.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return bool
+ * @return bool True if comments are available, false if no more comments.
  */
 function have_comments() {
 	global $wp_query;
@@ -928,9 +998,9 @@ function have_comments() {
  *
  * @since 2.2.0
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
- * @return object
+ * @return null
  */
 function the_comment() {
 	global $wp_query;
@@ -946,7 +1016,7 @@ function the_comment() {
  */
 function wp_old_slug_redirect() {
 	if ( is_404() && '' !== get_query_var( 'name' ) ) {
-		// Guess the current post_type based on the query vars.
+		// Guess the current post type based on the query vars.
 		if ( get_query_var( 'post_type' ) ) {
 			$post_type = get_query_var( 'post_type' );
 		} elseif ( get_query_var( 'attachment' ) ) {
@@ -964,7 +1034,7 @@ function wp_old_slug_redirect() {
 			$post_type = reset( $post_type );
 		}
 
-		// Do not attempt redirect for hierarchical post types
+		// Do not attempt redirect for hierarchical post types.
 		if ( is_post_type_hierarchical( $post_type ) ) {
 			return;
 		}
@@ -992,7 +1062,7 @@ function wp_old_slug_redirect() {
 
 		if ( get_query_var( 'paged' ) > 1 ) {
 			$link = user_trailingslashit( trailingslashit( $link ) . 'page/' . get_query_var( 'paged' ) );
-		} elseif( is_embed() ) {
+		} elseif ( is_embed() ) {
 			$link = user_trailingslashit( trailingslashit( $link ) . 'embed' );
 		}
 
@@ -1009,7 +1079,7 @@ function wp_old_slug_redirect() {
 			return;
 		}
 
-		wp_redirect( $link, 301 ); // Permanent redirect
+		wp_redirect( $link, 301 ); // Permanent redirect.
 		exit;
 	}
 }
@@ -1017,31 +1087,30 @@ function wp_old_slug_redirect() {
 /**
  * Find the post ID for redirecting an old slug.
  *
- * @see wp_old_slug_redirect()
- *
  * @since 4.9.3
  * @access private
  *
+ * @see wp_old_slug_redirect()
  * @global wpdb $wpdb WordPress database abstraction object.
  *
  * @param string $post_type The current post type based on the query vars.
- * @return int $id The Post ID.
+ * @return int The Post ID.
  */
 function _find_post_by_old_slug( $post_type ) {
 	global $wpdb;
 
 	$query = $wpdb->prepare( "SELECT post_id FROM $wpdb->postmeta, $wpdb->posts WHERE ID = post_id AND post_type = %s AND meta_key = '_wp_old_slug' AND meta_value = %s", $post_type, get_query_var( 'name' ) );
 
-	// if year, monthnum, or day have been specified, make our query more precise
-	// just in case there are multiple identical _wp_old_slug values
+	// If year, monthnum, or day have been specified, make our query more precise
+	// just in case there are multiple identical _wp_old_slug values.
 	if ( get_query_var( 'year' ) ) {
-		$query .= $wpdb->prepare( " AND YEAR(post_date) = %d", get_query_var( 'year' ) );
+		$query .= $wpdb->prepare( ' AND YEAR(post_date) = %d', get_query_var( 'year' ) );
 	}
 	if ( get_query_var( 'monthnum' ) ) {
-		$query .= $wpdb->prepare( " AND MONTH(post_date) = %d", get_query_var( 'monthnum' ) );
+		$query .= $wpdb->prepare( ' AND MONTH(post_date) = %d', get_query_var( 'monthnum' ) );
 	}
 	if ( get_query_var( 'day' ) ) {
-		$query .= $wpdb->prepare( " AND DAYOFMONTH(post_date) = %d", get_query_var( 'day' ) );
+		$query .= $wpdb->prepare( ' AND DAYOFMONTH(post_date) = %d', get_query_var( 'day' ) );
 	}
 
 	$id = (int) $wpdb->get_var( $query );
@@ -1052,28 +1121,27 @@ function _find_post_by_old_slug( $post_type ) {
 /**
  * Find the post ID for redirecting an old date.
  *
- * @see wp_old_slug_redirect()
- *
  * @since 4.9.3
  * @access private
  *
+ * @see wp_old_slug_redirect()
  * @global wpdb $wpdb WordPress database abstraction object.
  *
  * @param string $post_type The current post type based on the query vars.
- * @return int $id The Post ID.
+ * @return int The Post ID.
  */
 function _find_post_by_old_date( $post_type ) {
 	global $wpdb;
 
 	$date_query = '';
 	if ( get_query_var( 'year' ) ) {
-		$date_query .= $wpdb->prepare( " AND YEAR(pm_date.meta_value) = %d", get_query_var( 'year' ) );
+		$date_query .= $wpdb->prepare( ' AND YEAR(pm_date.meta_value) = %d', get_query_var( 'year' ) );
 	}
 	if ( get_query_var( 'monthnum' ) ) {
-		$date_query .= $wpdb->prepare( " AND MONTH(pm_date.meta_value) = %d", get_query_var( 'monthnum' ) );
+		$date_query .= $wpdb->prepare( ' AND MONTH(pm_date.meta_value) = %d', get_query_var( 'monthnum' ) );
 	}
 	if ( get_query_var( 'day' ) ) {
-		$date_query .= $wpdb->prepare( " AND DAYOFMONTH(pm_date.meta_value) = %d", get_query_var( 'day' ) );
+		$date_query .= $wpdb->prepare( ' AND DAYOFMONTH(pm_date.meta_value) = %d', get_query_var( 'day' ) );
 	}
 
 	$id = 0;
@@ -1081,7 +1149,7 @@ function _find_post_by_old_date( $post_type ) {
 		$id = (int) $wpdb->get_var( $wpdb->prepare( "SELECT post_id FROM $wpdb->postmeta AS pm_date, $wpdb->posts WHERE ID = post_id AND post_type = %s AND meta_key = '_wp_old_date' AND post_name = %s" . $date_query, $post_type, get_query_var( 'name' ) ) );
 
 		if ( ! $id ) {
-			// Check to see if an old slug matches the old date
+			// Check to see if an old slug matches the old date.
 			$id = (int) $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts, $wpdb->postmeta AS pm_slug, $wpdb->postmeta AS pm_date WHERE ID = pm_slug.post_id AND ID = pm_date.post_id AND post_type = %s AND pm_slug.meta_key = '_wp_old_slug' AND pm_slug.meta_value = %s AND pm_date.meta_key = '_wp_old_date'" . $date_query, $post_type, get_query_var( 'name' ) ) );
 		}
 	}
@@ -1095,7 +1163,7 @@ function _find_post_by_old_date( $post_type ) {
  * @since 1.5.0
  * @since 4.4.0 Added the ability to pass a post ID to `$post`.
  *
- * @global WP_Query $wp_query Global WP_Query instance.
+ * @global WP_Query $wp_query WordPress Query object.
  *
  * @param WP_Post|object|int $post WP_Post instance or Post ID/object.
  * @return bool True when finished.
@@ -1109,3 +1177,23 @@ function setup_postdata( $post ) {
 
 	return false;
 }
+
+/**
+ * Generates post data.
+ *
+ * @since 5.2.0
+ *
+ * @global WP_Query $wp_query WordPress Query object.
+ *
+ * @param WP_Post|object|int $post WP_Post instance or Post ID/object.
+ * @return array|bool Elements of post, or false on failure.
+ */
+function generate_postdata( $post ) {
+	global $wp_query;
+
+	if ( ! empty( $wp_query ) && $wp_query instanceof WP_Query ) {
+		return $wp_query->generate_postdata( $post );
+	}
+
+	return false;
+}
